'use strict';

var _Object$keys = require('babel-runtime/core-js/object/keys')['default'];

var _interopRequireDefault = require('babel-runtime/helpers/interop-require-default')['default'];

Object.defineProperty(exports, '__esModule', {
  value: true
});
exports['default'] = QueryAPIBabelPlugin;

var _minimatch = require('minimatch');

var _minimatch2 = _interopRequireDefault(_minimatch);

function QueryAPIBabelPlugin(_ref) {
  var Plugin = _ref.Plugin;
  var t = _ref.types;

  var requireSitegenAPIImpl = t.callExpression(t.identifier('require'), [t.literal('sitegen/internal')]);

  function makeRegExpLiteral(re) {
    return {
      type: 'Literal',
      regex: { pattern: re.source, flags: '' }
    };
  }

  function makeSitegenAPICall(name, args) {
    var callee = t.memberExpression(requireSitegenAPIImpl, t.identifier(name));
    return t.callExpression(callee, args);
  }

  function makeRequire(request) {
    return t.callExpression(t.identifier('require'), [t.literal(request)]);
  }

  function makeRequireContext(request, pattern) {
    var callee = t.memberExpression(t.identifier('require'), t.identifier('context'));
    return t.callExpression(callee, [t.literal(request), t.literal(true), makeRegExpLiteral(pattern)]);
  }

  return new Plugin('sitegen-query-api', {
    visitor: {

      CallExpression: function CallExpression(node, parent, scope, file) {

        if (!file.isSitegenAPI) {
          file.isSitegenAPI = makeSitegenAPIRecognizer(file);
        }

        if (file.isSitegenAPI('includePage', node, scope)) {
          checkSitegenAPI('includePage', node, file);
          var request = node.arguments[0].value;
          var moduleNode = makeRequire('page!' + request);
          return makeSitegenAPICall('wrapPageModule', [moduleNode]);
        } else if (file.isSitegenAPI('includePages', node, scope)) {
          checkSitegenAPI('includePages', node, file);
          var request = node.arguments[0].value;

          var _parseRequest = parseRequest(request);

          var directory = _parseRequest.directory;
          var regexp = _parseRequest.regexp;

          var contextNode = makeRequireContext('page!' + directory, regexp);
          return makeSitegenAPICall('wrapPageContext', [contextNode]);
        } else if (file.isSitegenAPI('link', node, scope)) {
          checkSitegenAPI('link', node, file);
          var request = node.arguments[0].value;
          var moduleNode = makeRequire('page-id!' + request);
          return makeSitegenAPICall('wrapPageLinkModule', [moduleNode]);
        } else if (file.isSitegenAPI('links', node, scope)) {
          checkSitegenAPI('links', node, file);
          var request = node.arguments[0].value;

          var _parseRequest2 = parseRequest(request);

          var directory = _parseRequest2.directory;
          var regexp = _parseRequest2.regexp;

          var contextNode = makeRequireContext('page-id!' + directory, regexp);
          return makeSitegenAPICall('wrapPageLinkContext', [contextNode]);
        } else if (file.isSitegenAPI('page', node, scope)) {
          checkSitegenAPI('page', node, file);
          var request = node.arguments[0].value;
          var moduleNode = makeRequire('page-id!' + request);
          return makeSitegenAPICall('wrapPageMetaModule', [moduleNode]);
        } else if (file.isSitegenAPI('pages', node, scope)) {
          checkSitegenAPI('pages', node, file);
          var request = node.arguments[0].value;

          var _parseRequest3 = parseRequest(request);

          var directory = _parseRequest3.directory;
          var regexp = _parseRequest3.regexp;

          var contextNode = makeRequireContext('page-id!' + directory, regexp);
          return makeSitegenAPICall('wrapPageMetaContext', [contextNode]);
        }
      }
    }
  });
}

function makeSitegenAPIRecognizer(file) {

  var namespaceBinding = null;
  var moduleBindings = {};
  var nameMapping = {};

  for (var i = 0; i < file.metadata.modules.imports.length; i++) {
    var imp = file.metadata.modules.imports[i];
    if (imp.source !== 'sitegen') {
      continue;
    }
    for (var j = 0; j < imp.specifiers.length; j++) {
      var specifier = imp.specifiers[j];
      if (specifier.kind === 'namespace') {
        namespaceBinding = file.scope.getOwnBinding(specifier.local);
      } else if (specifier.kind === 'named') {
        moduleBindings[specifier.local] = file.scope.getOwnBinding(specifier.local);
        nameMapping[specifier.imported] = specifier.local;
      }
    }
  }

  var moduleHasSitegenAPI = namespaceBinding !== null || _Object$keys(moduleBindings).length > 0;

  return function isSitegenAPI(name, node, scope) {
    if (!moduleHasSitegenAPI) {
      return false;
    }
    if (node.type !== 'CallExpression') {
      return false;
    }
    var callee = node.callee;
    if (callee.type === 'MemberExpression') {
      if (callee.object.type !== 'Identifier' || callee.property.type !== 'Identifier') {
        return false;
      }
      if (scope.getBinding(callee.object.name) !== namespaceBinding) {
        return false;
      }
      if (callee.property.name !== name) {
        return false;
      }
    } else if (callee.type === 'Identifier') {
      name = nameMapping[name];
      if (callee.name !== name) {
        return false;
      }
      if (scope.getBinding(callee.name) !== moduleBindings[callee.name]) {
        return false;
      }
    }
    return true;
  };
}

function parseRequest(request) {
  var pattern = new _minimatch2['default'].Minimatch(request);
  var directory = [];
  for (var i = 0; i < pattern.set[0].length; i++) {
    if (typeof pattern.set[0][i] === 'string') {
      directory.push(pattern.set[0][i]);
    } else {
      break;
    }
  }
  if (directory.length > 1) {
    directory = directory.join('/');
  }
  pattern = new _minimatch2['default'].Minimatch('./' + request.substring(directory.length + 1));
  var regexp = pattern.makeRe();
  return { directory: directory, regexp: regexp };
}

function checkSitegenAPI(name, node, file) {
  if (node.arguments.length !== 1) {
    throw file.errorWithNode(node, 'Sitegen.' + name + '(...): only single argument is allowed');
  }
  if (node.arguments[0].type !== 'Literal') {
    throw file.errorWithNode(node, 'Sitegen.' + name + '(...): only string literal argument is allowed');
  }
  if (node.arguments[0].type !== 'Literal') {
    throw file.errorWithNode(node, 'Sitegen.' + name + '(...): only string literal argument is allowed');
  }
  if (typeof node.arguments[0].type !== 'string') {
    throw file.errorWithNode(node, 'Sitegen.' + name + '(...): only string literal argument is allowed');
  }
}
module.exports = exports['default'];
